@@ -24,22 +24,33 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+/**
+ * SECTION:gtkmenubar
+ * @Title: GtkMenuBar
+ * @Short_description: A subclass of GtkMenuShell which holds GtkMenuItem widgets
+ * @See_also: #GtkMenuShell, #GtkMenu, #GtkMenuItem
+ *
+ * The #GtkMenuBar is a subclass of #GtkMenuShell which contains one or
+ * more #GtkMenuItems. The result is a standard menu bar which can hold
+ * many menu items.
+ */
+
 #include "config.h"
 
 #include "gtkmenubar.h"
 
-#include "gdk/gdkkeysyms.h"
 #include "gtkbindings.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
-#include "gtkmenuitem.h"
+#include "gtkmenuitemprivate.h"
 #include "gtkmenuprivate.h"
+#include "gtkmenushellprivate.h"
 #include "gtksettings.h"
 #include "gtksizerequest.h"
 #include "gtkwindow.h"
-
 #include "gtkintl.h"
 #include "gtkprivate.h"
+#include "gtktypebuiltins.h"
 
 #define BORDER_SPACING  0
 #define DEFAULT_IPADDING 1
@@ -214,11 +225,23 @@ gtk_menu_bar_class_init (GtkMenuBarClass *class)
 static void
 gtk_menu_bar_init (GtkMenuBar *menu_bar)
 {
+  GtkStyleContext *context;
+
   menu_bar->priv = G_TYPE_INSTANCE_GET_PRIVATE (menu_bar,
                                                 GTK_TYPE_MENU_BAR,
                                                 GtkMenuBarPrivate);
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (menu_bar));
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_MENUBAR);
 }
 
+/**
+ * gtk_menu_bar_new:
+ *
+ * Creates a new #GtkMenuBar
+ *
+ * Returns: the new menu bar, as a #GtkWidget
+ */
 GtkWidget*
 gtk_menu_bar_new (void)
 {
@@ -296,7 +319,7 @@ gtk_menu_bar_size_request (GtkWidget      *widget,
       priv = menu_bar->priv;
 
       nchildren = 0;
-      children = menu_shell->children;
+      children = menu_shell->priv->children;
 
       while (children)
 	{
@@ -307,7 +330,6 @@ gtk_menu_bar_size_request (GtkWidget      *widget,
 	    {
               gint toggle_size;
 
-	      GTK_MENU_ITEM (child)->show_submenu_indicator = FALSE;
               gtk_widget_get_preferred_size (child, &child_requisition, NULL);
               gtk_menu_item_toggle_size_request (GTK_MENU_ITEM (child),
                                                  &toggle_size);
@@ -345,11 +367,18 @@ gtk_menu_bar_size_request (GtkWidget      *widget,
 
       if (get_shadow_type (menu_bar) != GTK_SHADOW_NONE)
 	{
-          GtkStyle *style;
+          GtkStyleContext *context;
+          GtkBorder *border;
 
-          style = gtk_widget_get_style (widget);
-	  requisition->width += style->xthickness * 2;
-	  requisition->height += style->ythickness * 2;
+          context = gtk_widget_get_style_context (widget);
+
+          gtk_style_context_get (context, 0,
+                                 "border-width", &border,
+                                 NULL);
+
+	  requisition->width += border->left + border->right;
+	  requisition->height += border->top + border->bottom;
+          gtk_border_free (border);
 	}
     }
 }
@@ -413,7 +442,7 @@ gtk_menu_bar_size_allocate (GtkWidget     *widget,
 
   gtk_widget_style_get (widget, "internal-padding", &ipadding, NULL);
   
-  if (menu_shell->children)
+  if (menu_shell->priv->children)
     {
       border_width = gtk_container_get_border_width (GTK_CONTAINER (menu_bar));
       child_allocation.x = (border_width +
@@ -424,26 +453,33 @@ gtk_menu_bar_size_allocate (GtkWidget     *widget,
       
       if (get_shadow_type (menu_bar) != GTK_SHADOW_NONE)
 	{
-          GtkStyle *style;
+          GtkStyleContext *context;
+          GtkBorder *border;
 
-          style = gtk_widget_get_style (widget);
-          child_allocation.x += style->xthickness;
-          child_allocation.y += style->ythickness;
+          context = gtk_widget_get_style_context (widget);
+          gtk_style_context_get (context, 0,
+                                 "border-width", &border,
+                                 NULL);
+
+          child_allocation.x += border->left;
+          child_allocation.y += border->top;
+
+          gtk_border_free (border);
 	}
       
       if (priv->pack_direction == GTK_PACK_DIRECTION_LTR ||
 	  priv->pack_direction == GTK_PACK_DIRECTION_RTL)
 	{
 	  child_allocation.height = MAX (1, (gint)allocation->height - child_allocation.y * 2);
-	  
-	  offset = child_allocation.x; 	/* Window edge to menubar start */
-	  ltr_x = child_allocation.x;
-	  
-	  children = menu_shell->children;
-	  while (children)
-	    {
-	      gint toggle_size;          
-	      
+
+          offset = child_allocation.x; 	/* Window edge to menubar start */
+          ltr_x = child_allocation.x;
+
+          children = menu_shell->priv->children;
+          while (children)
+            {
+              gint toggle_size;
+
 	      child = children->data;
 	      children = children->next;
 	      
@@ -456,10 +492,11 @@ gtk_menu_bar_size_allocate (GtkWidget     *widget,
 		child_requisition.width += toggle_size;
 	      else
 		child_requisition.height += toggle_size;
-	      
+
 	      /* Support for the right justified help menu */
-	      if ((children == NULL) && (GTK_IS_MENU_ITEM(child))
-		  && (GTK_MENU_ITEM(child)->right_justify)) 
+	      if (children == NULL &&
+                  GTK_IS_MENU_ITEM (child) &&
+                  GTK_MENU_ITEM (child)->priv->right_justify)
 		{
 		  ltr_x = allocation->width -
 		    child_requisition.width - offset;
@@ -485,15 +522,15 @@ gtk_menu_bar_size_allocate (GtkWidget     *widget,
       else
 	{
 	  child_allocation.width = MAX (1, (gint)allocation->width - child_allocation.x * 2);
-	  
-	  offset = child_allocation.y; 	/* Window edge to menubar start */
-	  ltr_y = child_allocation.y;
-	  
-	  children = menu_shell->children;
-	  while (children)
-	    {
-	      gint toggle_size;          
-	      
+
+          offset = child_allocation.y; 	/* Window edge to menubar start */
+          ltr_y = child_allocation.y;
+
+          children = menu_shell->priv->children;
+          while (children)
+            {
+              gint toggle_size;
+
 	      child = children->data;
 	      children = children->next;
 	      
@@ -506,11 +543,12 @@ gtk_menu_bar_size_allocate (GtkWidget     *widget,
 		child_requisition.width += toggle_size;
 	      else
 		child_requisition.height += toggle_size;
-	      
-	      /* Support for the right justified help menu */
-	      if ((children == NULL) && (GTK_IS_MENU_ITEM(child))
-		  && (GTK_MENU_ITEM(child)->right_justify)) 
-		{
+
+              /* Support for the right justified help menu */
+              if (children == NULL &&
+                  GTK_IS_MENU_ITEM (child) &&
+                  GTK_MENU_ITEM (child)->priv->right_justify)
+                {
 		  ltr_y = allocation->height -
 		    child_requisition.height - offset;
 		}
@@ -539,18 +577,26 @@ static gint
 gtk_menu_bar_draw (GtkWidget *widget,
 		   cairo_t   *cr)
 {
+  GtkStyleContext *context;
+  GtkStateFlags state;
   int border;
 
   border = gtk_container_get_border_width (GTK_CONTAINER (widget));
+  context = gtk_widget_get_style_context (widget);
 
-  gtk_paint_box (gtk_widget_get_style (widget),
-                 cr,
-                 gtk_widget_get_state (widget),
-                 get_shadow_type (GTK_MENU_BAR (widget)),
-                 widget, "menubar",
-                 border, border,
-                 gtk_widget_get_allocated_width (widget) - border * 2,
-                 gtk_widget_get_allocated_height (widget) - border * 2);
+  state = gtk_widget_get_state_flags (widget);
+  gtk_style_context_set_state (context, state);
+
+  if (get_shadow_type (GTK_MENU_BAR (widget)) != GTK_SHADOW_NONE)
+    gtk_render_background (context, cr,
+                           border, border,
+                           gtk_widget_get_allocated_width (widget) - border * 2,
+                           gtk_widget_get_allocated_height (widget) - border * 2);
+
+  gtk_render_frame (context, cr,
+                    border, border,
+                    gtk_widget_get_allocated_width (widget) - border * 2,
+                    gtk_widget_get_allocated_height (widget) - border * 2);
 
   GTK_WIDGET_CLASS (gtk_menu_bar_parent_class)->draw (widget, cr);
 
@@ -640,7 +686,6 @@ window_key_press_handler (GtkWidget   *widget,
 	      GtkMenuShell *menu_shell = GTK_MENU_SHELL (menubars->data);
 
               _gtk_menu_shell_set_keyboard_mode (menu_shell, TRUE);
-	      _gtk_menu_shell_activate (menu_shell);
 	      gtk_menu_shell_select_first (menu_shell, FALSE);
 	      
 	      g_list_free (menubars);
@@ -739,8 +784,8 @@ _gtk_menu_bar_cycle_focus (GtkMenuBar       *menubar,
 	  if (current && current->next)
 	    {
 	      GtkMenuShell *new_menushell = GTK_MENU_SHELL (current->next->data);
-	      if (new_menushell->children)
-		to_activate = new_menushell->children->data;
+	      if (new_menushell->priv->children)
+		to_activate = new_menushell->priv->children->data;
 	    }
 	}
 	  
@@ -885,7 +930,7 @@ gtk_menu_bar_set_pack_direction (GtkMenuBar       *menubar,
 
       gtk_widget_queue_resize (GTK_WIDGET (menubar));
 
-      for (l = GTK_MENU_SHELL (menubar)->children; l; l = l->next)
+      for (l = GTK_MENU_SHELL (menubar)->priv->children; l; l = l->next)
 	gtk_widget_queue_resize (GTK_WIDGET (l->data));
 
       g_object_notify (G_OBJECT (menubar), "pack-direction");
@@ -938,7 +983,7 @@ gtk_menu_bar_set_child_pack_direction (GtkMenuBar       *menubar,
 
       gtk_widget_queue_resize (GTK_WIDGET (menubar));
 
-      for (l = GTK_MENU_SHELL (menubar)->children; l; l = l->next)
+      for (l = GTK_MENU_SHELL (menubar)->priv->children; l; l = l->next)
 	gtk_widget_queue_resize (GTK_WIDGET (l->data));
 
       g_object_notify (G_OBJECT (menubar), "child-pack-direction");
