@@ -24,14 +24,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
 #ifndef __GTK_SEPARATOR_MENU_ITEM_H__
 #define __GTK_SEPARATOR_MENU_ITEM_H__
 
-#include <gdk/gdk.h>
+
 #include <gtk/gtkmenuitem.h>
 
 
@@ -56,6 +56,12 @@ struct _GtkSeparatorMenuItem
 struct _GtkSeparatorMenuItemClass
 {
   GtkMenuItemClass parent_class;
+
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 
